//Scheduler event handler add-on public API header file

#ifndef _EVENT_ADD_ON_PUBLIC_H_
#define _EVENT_ADD_ON_PUBLIC_H_


//******************************************************************************************************
//**** SYSTEM HEADER FILES
//******************************************************************************************************
#include <TypeConstants.h>


//******************************************************************************************************
//**** PROJECT HEADER FILES AND CLASS NAME DECLARATIONS
//******************************************************************************************************
#include "EndianIndependentFile.h"	//Santa's gift bag
#include "SchedulerBuild.h"
#include "Event.h"


//******************************************************************************************************
//**** BMessage what codes
//******************************************************************************************************
const uint32 ProcessHoldDone = 'Sphd';


//******************************************************************************************************
//**** EventAddOnData CLASS DECLARATION
//******************************************************************************************************
class _SCHIMPEXP EventAddOnData
{
	public:
		EventAddOnData(Event* AttachTo);

		//Functions which must be implemented by derived classes
		virtual ~EventAddOnData();
			//Virtual destructor used to call derived classes' destructors.
		virtual BView* CreateEditView(BRect frame) = 0;
			//Create the view that is used to edit the event data.
			//If the user makes any changes, you should call GetAttachedTo()->SetModified();  Note that
			//when you do this, you need to wrap it in ReadWriteLock() and ReadWriteUnlock() calls to
			//the AssertableRWSemaphore returned by GetAttachedTo()->GetProtectionSemaphore();
		virtual bool SaveData(EndianIndependentFile* File) = 0;
			//Allows the add-on to store its data into the Event set file.  In case the add-on is
			//unavailable, the first thing that should be written is a 32-bit integer indicating the
			//length of the rest of the data in bytes (not including the 4 bytes for the size).  The
			//file will already contain data for other events, etc.  Should return true if everything
			//was OK, false if a file error occurred.
		virtual void DealWithEvent(bool IsEventLate) = 0;
			//This method may not block and may not consume much time.  Scheduler asserts that no DealWithEvent
			//call may take more than a second to aid in debugging add-ons.  After DealWithEvent returns, the
			//event will be updated and possibly deleted, so the time-consuming process should already have all
			//the information that it needs internally so that it is not dependent on the Event, the
			//EventAddOnData, or your class derived from EventAddOnData.  If your add-on's DealWithEvent
			//implementation is time consuming or blocks, you must spawn a thread to perform the task.  Before
			//spawning the thread, EventAddOnData::ProcessHold() must be called.  This keeps the app from
			//quitting while your add-on is doing its thing.  When the thread is done, you must send a
			//ProcessHoldDone message to the be_app object.  The event should be delivered in such a manner
			//as to indicate that it is late if IsEventLate is true.

		//Functions with fixed implementations
		virtual void DestroyEditView();				//Removes the edit view and its contents
		BView* GetEditView();						//Returns the edit view
		void RecordEditView(BView* TheEditView);	//Records the edit view
		Event* GetParentEvent();					//Returns the Event this is attached to
		static void ProcessHold();					//Tell the application that a time-consuming process is in progress
													//You must send be_app a ProcessHoldDone message when completed.
		Event* GetAttachedTo();

	private:
		//FBC protection
		virtual void EAODReserved1();
		virtual void EAODReserved2();
		virtual void EAODReserved3();
		virtual void EAODReserved4();
		virtual void EAODReserved5();
		virtual void EAODReserved6();
		int32 EAODReserved7;
		int32 EAODReserved8;
		int32 EAODReserved9;
		int32 EAODReserved10;
		int32 EAODReserved11;
		int32 EAODReserved12;

		BView* EditView;			//Edit view created by the add-on in the Event window (NOT the
									//event delivery window from DealWithEvent())
		Event* AttachedTo;
};


//******************************************************************************************************
//**** FUNCTION DECLARATIONS
//******************************************************************************************************
_SCHIMPEXP char* GenerateMessageAlertText(Event* AttachedTo,bool IsEventLate,const char* Message1,
	const char* Message2,const char* Message3,const char* Message4,const char* Message5);
	//Generates standard event message text for display in an alert.  The returned string must be deleted
	//by the caller.
_SCHIMPEXP void DeliverEventMessage(Event* AttachedTo,bool IsEventLate,const char* Message1,const char* Message2,
	const char* Message3,const char* Message4,const char* Message5);
	//This is a function that is quite useful.  It displays a dialog containing information about the event,
	//including time, if it was missed, etc. etc., nicely phrased, then displays the concatenated message
	//given in the five message strings (which may be NULL too), along with an ok button.  This function does
	//not block so it is safe to call directly from DealWithEvent().  When the event is completed, it sends a
	//ProcessHoldDone message to be_app.

_SCHIMPEXP status_t WriteAddOnSetting(const char* SettingName,const void* data,size_t size,
	uint32 type = B_RAW_TYPE);
_SCHIMPEXP status_t ReadAddOnSetting(const char* SettingName,const void*& out_data,ssize_t& out_size,
	uint32& out_type);
	//These functions let you store settings along with Scheduler's settings.  The calling conventions match
	//those of Jon Watte's libprefs, which Scheduler actually uses.  To prevent naming conflicts, you should
	//prepend your SettingName with the name of your add-on, for example "MyAddOnName_SomeSettingName".


//******************************************************************************************************
//**** FUNCTIONS WHICH MUST BE EXPORTED BY ADD-ONS
//******************************************************************************************************

/* These functions need to be exported by your add-on:
extern "C"
{
	__declspec(dllexport) const char* InitializeAddOn();
	__declspec(dllexport) EventAddOnData* NewAddOnData(Event* AttachTo, EndianIndependentFile* LoadFile,
		bool* FileOK);
	__declspec(dllexport) void CleanUpAddOn();
}

const char* InitializeAddOn()
	Called as soon as each add-on image is loaded.  Doesn't necessarily have to do anything except return
	the name of the add-on.  Returning NULL indicates that an error occurred and the add-on should be
	unloaded immediately.  The following names are taken by the default event handlers: "Unknown",
	"Do nothing","Message","Command","Shell script","Act on file".  Please check with me (btietz@ucsd.edu)
	before choosing a name so I can make sure that no two third-party add-ons are given the same name.
	If your initialization fails, return NULL and your add-on will be ignored by Scheduler.

EventAddOnData* NewAddOnData(Event* AttachTo,EndianIndependentFile* LoadFile,bool* FileOK)
	Called to create a new instance of add-on data.  If file is NULL, the event should be created as
	untitled with default values.  Otherwise, File will currently be at the 32-bit size value that the
	add-on previously wrote in EventAddOnData::SaveData().  FileOK is used to return to the caller
	whether everything went file with the file.  true indicates everything was fine, false indicates
	that a file error occurred.

void CleanUpAddOn()
	Called just before each add-on image is unloaded.


*/


#endif
